"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Worker = void 0;
const util_1 = require("@cardano-graphql/util");
const object_hash_1 = __importDefault(require("object-hash"));
const ts_log_1 = require("ts-log");
const pg_boss_1 = __importDefault(require("pg-boss"));
const ASSET_METADATA_FETCH_INITIAL = 'asset-metadata-fetch-initial';
const ASSET_METADATA_FETCH_UPDATE = 'asset-metadata-fetch-update';
const SIX_HOURS = 21600;
const MODULE_NAME = 'Worker';
class Worker {
    constructor(hasuraClient, logger = ts_log_1.dummyLogger, metadataFetchClient, queueConfig, options) {
        this.hasuraClient = hasuraClient;
        this.logger = logger;
        this.metadataFetchClient = metadataFetchClient;
        this.options = options;
        this.state = 'initialized';
        this.queue = new pg_boss_1.default(Object.assign({ application_name: 'cardano-graphql' }, queueConfig));
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== 'initialized') {
                throw new util_1.errors.ModuleIsNotInitialized(MODULE_NAME, 'start');
            }
            this.logger.info({ module: MODULE_NAME }, 'Starting');
            const subscriptionHandler = (data) => __awaiter(this, void 0, void 0, function* () {
                var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
                if ('length' in data) {
                    const jobs = data;
                    this.logger.debug({ module: MODULE_NAME, qty: jobs.length }, 'Processing jobs');
                    const assetIds = jobs.map((job) => job.data.assetId);
                    const fetchedMetadata = yield this.metadataFetchClient.fetch(assetIds);
                    const existingAssetMetadataHashes = yield this.hasuraClient.getAssetMetadataHashesById(assetIds);
                    for (const job of jobs) {
                        const assetId = job.data.assetId;
                        const metadata = fetchedMetadata.find(item => item.subject === assetId);
                        if (metadata === undefined) {
                            this.logger.trace({ module: MODULE_NAME, assetId }, 'Metadata not found in registry. Will retry');
                            job.done(new Error(`Metadata for asset ${assetId} not found in registry`));
                        }
                        else {
                            const existingAssetMetadataHashObj = existingAssetMetadataHashes.find(item => item.assetId === assetId);
                            const metadataHash = object_hash_1.default(metadata);
                            if ((existingAssetMetadataHashObj === null || existingAssetMetadataHashObj === void 0 ? void 0 : existingAssetMetadataHashObj.metadataHash) === metadataHash) {
                                this.logger.trace({ module: MODULE_NAME, assetId }, 'Metadata from registry matches local');
                            }
                            else {
                                this.logger.trace({ module: MODULE_NAME, assetId }, 'Found metadata in registry');
                                yield this.hasuraClient.addAssetMetadata({
                                    assetId,
                                    decimals: (_a = metadata.decimals) === null || _a === void 0 ? void 0 : _a.value,
                                    description: (_b = metadata.description) === null || _b === void 0 ? void 0 : _b.value,
                                    logo: (_c = metadata.logo) === null || _c === void 0 ? void 0 : _c.value,
                                    name: (_d = metadata.name) === null || _d === void 0 ? void 0 : _d.value,
                                    ticker: (_e = metadata.ticker) === null || _e === void 0 ? void 0 : _e.value,
                                    url: (_f = metadata.url) === null || _f === void 0 ? void 0 : _f.value,
                                    metadataHash
                                });
                                yield this.queue.publishAfter(ASSET_METADATA_FETCH_UPDATE, { assetId }, {
                                    retryDelay: (_j = (_h = (_g = this.options) === null || _g === void 0 ? void 0 : _g.metadataUpdateInterval) === null || _h === void 0 ? void 0 : _h.assets) !== null && _j !== void 0 ? _j : SIX_HOURS
                                }, (_m = (_l = (_k = this.options) === null || _k === void 0 ? void 0 : _k.metadataUpdateInterval) === null || _l === void 0 ? void 0 : _l.assets) !== null && _m !== void 0 ? _m : SIX_HOURS);
                            }
                        }
                    }
                }
            });
            yield this.queue.start();
            yield this.queue.subscribe(ASSET_METADATA_FETCH_INITIAL, {
                batchSize: 500,
                newJobCheckIntervalSeconds: 5
            }, subscriptionHandler);
            yield this.queue.subscribe(ASSET_METADATA_FETCH_UPDATE, {
                batchSize: 500,
                newJobCheckIntervalSeconds: 5
            }, subscriptionHandler);
            this.logger.info({ module: MODULE_NAME }, 'Started');
        });
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== 'running') {
                throw new util_1.errors.ModuleIsNotInitialized(MODULE_NAME, 'shutdown');
            }
            this.logger.info({ module: MODULE_NAME }, 'Shutting down');
            yield Promise.all([
                this.queue.unsubscribe(ASSET_METADATA_FETCH_INITIAL),
                this.queue.unsubscribe(ASSET_METADATA_FETCH_UPDATE)
            ]);
            this.state = 'initialized';
            this.logger.info({ module: MODULE_NAME }, 'Shutdown complete');
        });
    }
}
exports.Worker = Worker;
//# sourceMappingURL=Worker.js.map